"""Concurrently download metadata for items on Archive.org.

usage:
    ia mine <itemlist.txt> [options...]

options:
    -h, --help  
    -c, --cache                 Write item metadta to a file called <identifier>_meta.json
    -o, --output=<output.json>  Write all metadata to a single output file <itemlist>.json
    -w, --workers=<count>       The number requests to run concurrently [default: 20].

"""
from sys import stdout, exit
from json import dumps

from docopt import docopt

from internetarchive import get_data_miner



# ia_mine()
#_________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)

    identifiers = [i.strip() for i in open(args['<itemlist.txt>'])]
    workers = int(args.get('--workers', 20)[0])
    miner = get_data_miner(identifiers, workers=workers)

    # If writing all metadata to a single output file, make sure that 
    # file is empty.
    if args['--output']:
        open(args['--output'][0], 'w').close()

    for i, item in miner.items():
        metadata = dumps(item.metadata)
        if args['--cache']:
            stdout.write('saving metadata for: {0}\n'.format(item.identifier))
            with open('{0}_meta.json'.format(item.identifier), 'w') as fp:
                fp.write(metadata)
        elif args['--output']:
            stdout.write('saving metadata for: {0}\n'.format(item.identifier))
            with open(args['--output'][0], 'a+') as fp:
                fp.write(metadata + '\n')
        else:
            stdout.write(metadata + '\n')
    exit(0)



"""Search the Internet Archive using the Archive.org Advanced Search 
API <https://archive.org/advancedsearch.php#raw>.

usage: 
    ia search [--help] <query>... [options...]

options:
    -h, --help
    -p, --parameters=<key:value>...  Parameters to send with your query.
    -s, --sort=<field:order>...      Sort search results by specified
                                     fields. <order> can be either "asc"
                                     for ascending and "desc" for 
                                     descending.
    -f, --field=<field>...           Metadata fields to return.

"""
from docopt import docopt
import sys

from internetarchive import search



# main()
#_________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)

    params = dict(p.split(':') for p in args['--parameters'])
    if args['--sort']:
        for i, field in enumerate(args['--sort']):
            key = 'sort[{0}]'.format(i)
            params[key] = field.strip().replace(':', ' ')

    fields = ['identifier'] + args['--field']

    query = ' '.join(args['<query>'])
    search_resp = search(query, fields=fields, params=params)
    for result in search_resp.results():
        output = '\t'.join([result.get(f, '') for f in fields]).encode('utf-8')
        sys.stdout.write(output + '\n')
    sys.exit(0)



"""Retrieve information about your catalog tasks.

usage: 
    ia catalog [--help] [options...]

options:
    -h, --help
    -v, --verbose       
    -u, --url=<url>             ...
    -g, --green-rows            Return information about tasks that have not run.
    -b, --blue-rows             Return information about running tasks.
    -r, --red-rows              Return information about tasks that have failed.
    -f , --fields <field>...    Return only the specified fields. <field> may be one
                                of the following: 
                                
                                identifier, server, command, time, submitter, args, 
                                task_id, row_type
"""
from docopt import docopt
import sys
import urlparse

import internetarchive



# ia_catalog()
#_________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)

    if args['--url']:
        parsed_url = urlparse.urlparse(args['--url'][0])
        params = urlparse.parse_qs(parsed_url.query)
        c = internetarchive.Catalog(params=params)
    else:
        c = internetarchive.Catalog()
    row_types = {
            c.GREEN: 'green',
            c.BLUE: 'blue',
            c.RED: 'red',
            c.BROWN: 'brown',
    }
    if args['--green-rows']:
        tasks = c.green_rows
    elif args['--blue-rows']:
        tasks = c.blue_rows
    elif args['--red-rows']:
        tasks = c.red_rows
    else:
        tasks = c.tasks
    for t in tasks:
        if args['--fields']:
            task_info = []
            for field in args['--fields']:
                info = eval('t.{0}'.format(field))
                task_info.append(info)
        else:
            task_info = [
                    t.identifier, t.task_id, t.server, t.time, t.command, 
                    row_types[t.row_type]
            ]
            if args['--verbose']:
                targs = '\t'.join(['{0}={1}'.format(k, v) for (k,v) in t.args.items()])
                task_info += [t.submitter, targs]
        sys.stdout.write('\t'.join([str(x) for x in task_info]) + '\n')
    sys.exit(0)



from collections import defaultdict



# get_args_dict()
#_________________________________________________________________________________________
def get_args_dict(args):
    metadata = defaultdict(list)
    for md in args:
        key, value = md.split(':', 1)
        metadata[key].append(value)
    # Flatten single item lists.
    for key, value in metadata.items():
        if len(value) <= 1:
            metadata[key] = value[0]
    return metadata



"""Download files from archive.org.

usage: 
    ia download [--help] <identifier> [<file>...] [options...]  

options:
    -h, --help  
    --version  
    -i, --ignore-existing      Clobber files already downloaded.
    -s, --source=<source>...   Only Download files matching given sources.
    -o, --original             Download only files with source=original.
    -g, --glob=<pattern>       Only download files whose filename matches the 
                               given glob pattern.
    -f, --format=<format>...   Only download files of the specified format(s).
                               You can use the following command to retrieve
                               a list of file formats contained within a given 
                               item:
                                
                                   ia metadata --formats <identifier>

    -c, --concurrent           Download files concurrently using the Python 
                               gevent networking library.

"""
from docopt import docopt
import sys
import os

import internetarchive



# ia_download()
#_________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)

    if '/' in args['<identifier>']:
        identifier = args['<identifier>'].split('/')[0]
    else:
        identifier = args['<identifier>']
    item = internetarchive.Item(identifier)

    if '/' in args['<identifier>'] or args['<file>']:
        if not args['<file>']:
            fname = args['<identifier>'].split('/')[-1]
            files = [fname]
        else:
            files = args['<file>']
        for f in files:
            fname = f.encode('utf-8')
            path = os.path.join(identifier, fname)
            sys.stdout.write('downloading: {0}\n'.format(fname))
            f = item.file(fname)
            f.download(file_path=path, ignore_existing=args['--ignore-existing'])
        sys.exit(0)

    if args['--format']:
        formats = args['--format']
    else:
        formats = None

    if args['--glob']:
        glob = args['--glob'][0]
    else:
        glob = None

    if args['--source']:
        ia_source = args['--source']
    elif args['--original']:
        ia_source = ['original']
    else:
        ia_source = None

    item.download(formats=formats, source=ia_source, concurrent=args['--concurrent'], 
                  glob_pattern=glob, ignore_existing=args['--ignore-existing'])
    sys.exit(0)



try:
    import ujson as json
except ImportError:
    import json
import urllib
import urllib2

from . import config



# Search class
#_________________________________________________________________________________________
class Search(object):
    """This class represents an archive.org item search. You can use 
    this class to search for archive.org items using the advanced 
    search engine.

    Usage::

        >>> import internetarchive
        >>> search = internetarchive.Search('(uploader:jake@archive.org)')
        >>> for result in search.results:
        ...     print result['identifier']

    """

    # init()
    #_____________________________________________________________________________________
    def __init__(self, query, fields=['identifier'], params={}):
        self._base_url = 'http://archive.org/advancedsearch.php'
        self.query = query
        self.params = dict(dict(
                q = self.query,
                output = params.get('output', 'json'),
                rows = 100,
        ).items() + params.items())
        # Updata params dict with fields.
        for k, v in enumerate(fields):
            key = 'fl[{0}]'.format(k)
            self.params[key] = v
        self.encoded_params = urllib.urlencode(self.params)
        self.search_info = self._get_search_info()
        self.num_found = self.search_info['response']['numFound']


    # __repr__()
    #_____________________________________________________________________________________
    def __repr__(self):
        return ('Search(query={query!r}, '
                'num_found={num_found!r})'.format(**self.__dict__))


    # _get_search_info()
    #_____________________________________________________________________________________
    def _get_search_info(self):
        info_params = self.params.copy()
        info_params['rows'] = 0
        encoded_info_params = urllib.urlencode(info_params)
        f = urllib.urlopen(self._base_url, encoded_info_params)
        results = json.loads(f.read())
        del results['response']['docs']
        return results


    # _iter_results()
    #_____________________________________________________________________________________
    def results(self):
        """Generator for iterating over search results"""
        total_pages = ((self.num_found / self.params['rows']) + 2)
        for page in range(1, total_pages):
            self.params['page'] = page
            encoded_params = urllib.urlencode(self.params)
            f = urllib.urlopen(self._base_url, encoded_params)
            results = json.loads(f.read())
            for doc in results['response']['docs']:
                yield doc


#_________________________________________________________________________________________
class Catalog(object):
    """:todo: Document Catalog Class."""
    GREEN = 0
    BLUE = 1
    RED = 2
    BROWN = 9

    # init()
    #_____________________________________________________________________________________
    def __init__(self, params=None):
        url = 'http://archive.org/catalog.php'
        if params is None:
            params = dict(justme = 1)

        # Add params required to retrieve JSONP from the IA catalog.
        params['json'] = 2
        params['output'] = 'json'
        params['callback'] = 'foo'
        params = urllib.urlencode(params)

        logged_in_user, logged_in_sig = config.get_cookies()
        cookies = ('logged-in-user={0}; '
                   'logged-in-sig={1}; '
                   'verbose=1'.format(logged_in_user, logged_in_sig))

        opener = urllib2.build_opener()
        opener.addheaders.append(('Cookie', cookies))
        f = opener.open(url, params)

        # Convert JSONP to JSON (then parse the JSON).
        jsonp_str = f.read()
        json_str = jsonp_str[(jsonp_str.index("(") + 1):jsonp_str.rindex(")")]

        tasks_json = json.loads(json_str)
        self.tasks = [CatalogTask(t) for t in tasks_json]
        

    # filter_tasks()
    #_____________________________________________________________________________________
    def filter_tasks(self, pred):
        return [t for t in self.tasks if pred(t)]


    # tasks_by_type()
    #_____________________________________________________________________________________
    def tasks_by_type(self, row_type):
        return self.filter_tasks(lambda t: t.row_type == row_type)

    # green_rows()
    #_____________________________________________________________________________________
    @property
    def green_rows(self):
        return self.tasks_by_type(self.GREEN)


    # blue_rows()
    #_____________________________________________________________________________________
    @property
    def blue_rows(self):
        return self.tasks_by_type(self.BLUE)


    # red_rows()
    #_____________________________________________________________________________________
    @property
    def red_rows(self):
        return self.tasks_by_type(self.RED)


    # brown_rows()
    #_____________________________________________________________________________________
    @property
    def brown_rows(self):
        return self.tasks_by_type(self.BROWN)


# CatalogTask class
#_________________________________________________________________________________________
class CatalogTask(object):
    """represents catalog task.
    """
    COLUMNS = ('identifier', 'server', 'command', 'time', 'submitter',
               'args', 'task_id', 'row_type')

    def __init__(self, columns):
        """:param columns: array of values, typically returned by catalog
        web service. see COLUMNS for the column name.
        """
        for a, v in map(None, self.COLUMNS, columns):
            if a: setattr(self, a, v)
        # special handling for 'args' - parse it into a dict if it is a string
        if isinstance(self.args, basestring):
            self.args = dict(x for x in urllib2.urlparse.parse_qsl(self.args))

    def __repr__(self):
        return ('CatalogTask(identifier={identifier},'
                ' task_id={task_id!r}, server={server!r},'
                ' command={command!r},'
                ' submitter={submitter!r},'
                ' row_type={row_type})'.format(**self.__dict__))

    def __getitem__(self, k):
        """dict-like access privided as backward compatibility."""
        if k in self.COLUMNS:
            return getattr(self, k, None)
        else:
            raise KeyError, k

    def open_task_log(self):
        """return file-like reading task log."""
        if self.task_id is None:
            raise ValueError, 'task_id is None'
        url = 'http://catalogd.archive.org/log/{0}'.format(self.task_id)
        return urllib2.urlopen(url)



"""Upload files to Archive.org via the Internet Archive's S3 like server API. 

IA-S3 Documentation: https://archive.org/help/abouts3.txt

usage: 
    ia upload <identifier> [<file>...|-] [options...]

options:

 -h, --help
 -v, --verbose                  Print upload status to stdout.
 -d, --debug                    Print S3 request parameters to stdout and exit. [default: True]
 -r, --remote-name=<name>       When uploading data from stdin, this option sets the remote filename.
 -m, --metadata=<key:value>...  Metadata to add to your item.
 -H, --header=<key:value>...    Valid S3 HTTP headers to send with your request.
 -n, --no-derive                Do not derive uploaded files.
 -M, --multipart                Upload files to archive.org in parts, using multipart.
 -i, --ignore-bucket            Destroy and respecify all metadata. [default: True]

"""
from sys import stdin, stdout, stderr, exit
from tempfile import TemporaryFile
from xml.dom.minidom import parseString

from docopt import docopt

from internetarchive import upload
from iacli.argparser import get_args_dict



# get_xml_text()
#_________________________________________________________________________________________
def get_xml_text(elements, text=''):
    """:todo: document ``get_xml_text()`` function."""
    for e in elements:
        for node in e.childNodes:
            if node.nodeType == node.TEXT_NODE:
                text += node.data
    return text


# main()
#_________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)

    if args['--verbose'] and not args['--debug']:
        stdout.write('getting item: {0}\n'.format(args['<identifier>']))

    metadata = get_args_dict(args['--metadata'])
    s3_headers = get_args_dict(args['--header'])

    upload_kwargs = dict(
            metadata=metadata, 
            headers=s3_headers, 
            debug=args['--debug'], 
            queue_derive=args['--no-derive'], 
            ignore_bucket=args['--ignore-bucket'],
            verbose=args['--verbose'],
    )

    if args['<file>'] == ['-']:
        local_file = TemporaryFile()
        local_file.write(stdin.read())
        local_file.seek(0)
        upload_kwargs['remote_name'] = args['--remote-name'][0]
    else:
        local_file = args['<file>']

    response = upload(args['<identifier>'], local_file, **upload_kwargs)

    if args['--debug']:
        for r in response:
            headers = '\n'.join([' {0}: {1}'.format(k,v) for (k,v) in r.headers.items()])
            stdout.write('---\n\nEndpoint:\n {0}\n\n'.format(r.url))
            stdout.write('HTTP Headers:\n{0}\n\n'.format(headers))
    else:
        for resp in response:
            if resp.status_code == 200:
                continue
            error = parseString(resp.content)
            code = get_xml_text(error.getElementsByTagName('Code'))
            msg = get_xml_text(error.getElementsByTagName('Message'))
            stderr.write('error "{0}" ({1}): {2}\n'.format(code, resp.status_code, msg))
            exit(1)



"""
iacli library
~~~~~~~~~~~~~~~~~~~~~~~

Iacli is the CLI driver for the internetarchive python library:

<https://github.com/jjjake/ia-wrapper>.

:copyright: (c) 2013 by Jacob M. Johnson.
:license: GPL, see LICENSE for more details.

"""

from internetarchive import __version__



__title__ = 'iacli'
__version__ = __version__
__author__ = 'Jacob M. Johnson'
__license__ = 'GPL'
__copyright__ = 'Copyright 2013 Jacob M. Johnson'



import requests
from requests.exceptions import ConnectionError



# get_item_metadata()
#_____________________________________________________________________________________
def get_item_metadata(identifier, target=None, timeout=None, secure=False):
    """Get an item's metadata from the `Metadata API 
    <http://blog.archive.org/2013/07/04/metadata-api/>`__

    :type identifier: str
    :param identifier: Globally unique Archive.org identifier.

    :type target: bool
    :param target: (optional) Metadata target to retrieve.

    :type timeout: int
    :param timeout: (optional) Set the timeout for the HTTP request.  
                    Note: this is usually not needed as the Metadata API 
                    is designed to timeout if any connection issues 
                    arrise.

    :type secure: bool
    :param secure: (optional) If secure is True, use HTTPS protocol, 
                   otherwise use HTTP.

    :rtype: dict
    :returns: Metadat API response.

    """
    url = 'http://archive.org/metadata/{0}'.format(identifier)
    if secure:
        url = url.replace('http', 'https')
    r = requests.get(url)
    if r.status_code != 200:
        raise ConnectionError("Unable connect to Archive.org "
                              "({0})".format(r.status_code))
    metadata = r.json()
    if target:
        metadata = metadata.get(target, {})
    return metadata



import os

import yaml



# _get_config()
#_____________________________________________________________________________________
def _get_config(config={}):
    home_dir = os.environ.get('HOME')
    if not home_dir:
        return config
    config_file = os.path.join(home_dir, '.config', 'internetarchive.yml')
    try:
        config = yaml.load(open(config_file));
    except IOError:
        config_file = os.path.join(home_dir, '.internetarchive.yml')
        try:
            config = yaml.load(open(config_file))
        except IOError:
            return config
    return config


# get_s3_keys()
#_____________________________________________________________________________________
def get_s3_keys():
    access_key = os.environ.get('AWS_ACCESS_KEY_ID')
    secret_key = os.environ.get('AWS_SECRET_ACCESS_KEY')
    if not access_key or not secret_key:
        config = _get_config()
        s3_config = config.get('s3', {})
        access_key = s3_config.get('access_key')
        secret_key = s3_config.get('secret_key')
    return (access_key, secret_key)


# get_cookies()
#_____________________________________________________________________________________
def get_cookies():
    logged_in_user = os.environ.get('IA_LOGGED_IN_USER')
    logged_in_sig = os.environ.get('IA_LOGGED_IN_SIG')
    if not logged_in_user or not logged_in_sig:
        config = _get_config()
        cookie_config = config.get('cookies', {})
        logged_in_user = cookie_config.get('logged-in-user')
        logged_in_sig = cookie_config.get('logged-in-sig')
    return (logged_in_user, logged_in_sig)



# -*- coding: utf-8 -*-
"""
Internetarchive Library
~~~~~~~~~~~~~~~~~~~~~~~

Internetarchive is a python interface to archive.org.
usage:

    >>> import internetarchive
    >>> item = internetarchive.Item('govlawgacode20071')
    >>> item.exists
    True

:copyright: (c) 2013 by Jacob M. Johnson.
:license: GPL, see LICENSE for more details.

"""

__title__ = 'internetarchive'
__version__ = '0.3.9'
__author__ = 'Jacob M. Johnson'
__license__ = 'GPL'
__copyright__ = 'Copyright 2013 Jacob M. Johnson'

from .item import Item, File
from .service import Search, Catalog
#from .mine import Mine
from .api import *



try:
    import ujson as json
except ImportError:
    import json
import urllib
import os
from sys import stdout
import httplib
import urllib2
from fnmatch import fnmatch
from requests import Request, Session
from contextlib import closing

from jsonpatch import make_patch

from . import ias3, config, utils



# Item class
#_________________________________________________________________________________________
class Item(object):
    """This class represents an archive.org item. You can use this
    class to access item metadata::

        >>> import internetarchive
        >>> item = internetarchive.Item('stairs')
        >>> print item.metadata

    Or to modify the metadata for an item::

        >>> metadata = dict(title='The Stairs'))
        >>> item.modify(metadata)
        >>> print item.metadata['metadata']['title']
        u'The Stairs'

    This class also uses IA's S3-like interface to upload files to an
    item. You need to supply your IAS3 credentials in environment
    variables in order to upload::

        >>> import os
        >>> os.environ['AWS_ACCESS_KEY_ID'] = 'Y6oUrAcCEs4sK8ey'
        >>> os.environ['AWS_SECRET_ACCESS_KEY'] = 'youRSECRETKEYzZzZ'
        >>> item.upload('myfile.tar')
        True

    You can retrieve S3 keys here: `https://archive.org/account/s3.php
    <https://archive.org/account/s3.php>`__

    """
    # init()
    #_____________________________________________________________________________________
    def __init__(self, identifier, metadata_timeout=None, secure=False):
        """
        :type identifier: str
        :param identifier: The globally unique Archive.org identifier
                           for a given item.

        :type metadata_timeout: int
        :param metadata_timeout: (optional) Set a timeout for retrieving 
                                 an item's metadata.

        :type secure: bool
        :param secure: (optional) If secure is True, use HTTPS protocol, 
                       otherwise use HTTP.

        """
        self.identifier = identifier
        self.secure = secure
        if secure:
            protocol = 'https'
        else:
            protocol = 'http'
        self.details_url = '{0}://archive.org/details/{1}'.format(protocol, identifier)
        self.download_url = '{0}://archive.org/download/{1}'.format(protocol, identifier)
        self.metadata_url = '{0}://archive.org/metadata/{1}'.format(protocol, identifier)
        self.metadata_timeout = metadata_timeout
        self.session = None
        self.bucket = None
        self.metadata = utils.get_item_metadata(identifier, metadata_timeout, secure)
        if self.metadata == {}:
            self.exists = False
        else:
            self.exists = True


    # __repr__()
    #_____________________________________________________________________________________
    def __repr__(self):
        item_description = dict(
                    identifier = self.identifier,
                    exists = self.exists,
                    item_size = self.metadata.get('item_size'),
                    files_count = self.metadata.get('files_count'),
        )
        return ('Item(identifier={identifier!r}, '
                'exists={exists!r}, '
                'item_size={item_size!r}, '
                'files_count={files_count!r})'.format(**item_description))


    # files()
    #_____________________________________________________________________________________
    def files(self):
        """Generator for iterating over files in an item.

        :rtype: generator
        :returns: A generator that yields :class:`internetarchive.File
                  <File>` objects.

        """
        for file_dict in self.metadata.get('files', []):
            file = File(self, file_dict)
            yield file


    # file()
    #_____________________________________________________________________________________
    def file(self, name):
        """Get a :class:`File <File>` object for the named file.

        :rtype: :class:`internetarchive.File <File>`
        :returns: An :class:`internetarchive.File <File>` object.

        """
        for file_dict in self.metadata.get('files', []):
            if file_dict.get('name') == name:
                return File(self, file_dict)


    # download()
    #_____________________________________________________________________________________
    def download(self, concurrent=False, source=None, formats=None, glob_pattern=None,
                 ignore_existing=False):
        """Download the entire item into the current working directory.

        :type concurrent: bool
        :param concurrent: Download files concurrently if ``True``.

        :type source: str
        :param source: Only download files matching given source.

        :type formats: str
        :param formats: Only download files matching the given Formats.

        :type glob_pattern: str
        :param glob_pattern: Only download files matching the given glob
                             pattern

        :type ignore_existing: bool
        :param ignore_existing: Overwrite local files if they already
                                exist.

        :rtype: bool
        :returns: True if if files have been downloaded successfully.

        """
        if concurrent:
            try:
                from gevent import monkey
                monkey.patch_socket()
                from gevent.pool import Pool
                pool = Pool()
            except ImportError:
                raise ImportError(
                """No module named gevent

                Downloading files concurrently requires the gevent neworking library.
                gevent and all of it's dependencies can be installed with pip:

                \tpip install cython git+git://github.com/surfly/gevent.git@1.0rc2#egg=gevent

                """)

        files = self.files()
        if source:
            if type(source) == str:
                source = [source]
            files = [f for f in files if f.source in source]
        if formats:
            if type(formats) == str:
                formats = [formats]
            files = [f for f in files if f.format in formats]
        if glob_pattern:
            files = [f for f in files if fnmatch(f.name, glob_pattern)]

        for f in files:
            fname = f.name.encode('utf-8')
            path = os.path.join(self.identifier, fname)
            stdout.write('downloading: {0}\n'.format(fname))
            if concurrent:
                pool.spawn(f.download, path, ignore_existing=ignore_existing)
            else:
                f.download(path, ignore_existing=ignore_existing)
        if concurrent:
            pool.join()
        return True


    # modify_metadata()
    #_____________________________________________________________________________________
    def modify_metadata(self, metadata, target='metadata'):
        """Modify the metadata of an existing item on Archive.org.

        Note: The Metadata Write API does not yet comply with the
        latest Json-Patch standard. It currently complies with `version 02
        <https://tools.ietf.org/html/draft-ietf-appsawg-json-patch-02>`__.

        :type metadata: dict
        :param metadata: Metadata used to update the item.

        :type target: str
        :param target: (optional) Set the metadata target to update.

        Usage::

            >>> import internetarchive
            >>> item = internetarchive.Item('mapi_test_item1')
            >>> md = dict(new_key='new_value', foo=['bar', 'bar2'])
            >>> item.modify_metadata(md)

        :rtype: dict
        :returns: A dictionary containing the status_code and response
                  returned from the Metadata API.

        """
        access_key, secret_key = config.get_s3_keys()
        src = self.metadata.get(target, {})
        dest = dict((src.items() + metadata.items()))

        # Prepare patch to remove metadata elements with the value: "REMOVE_TAG".
        for k,v in metadata.items():
            if v == 'REMOVE_TAG' or not v:
                del dest[k]

        json_patch = make_patch(src, dest).patch
        # Reformat patch to be compliant with version 02 of the Json-Patch standard.
        patch = []
        for p in json_patch:
            pd = {p['op']: p['path']}
            if p['op'] != 'remove':
                pd['value'] = p['value']
            patch.append(dict((k,v) for k,v in pd.items() if v))

        data = {
            '-patch': json.dumps(patch),
            '-target': target,
            'access': access_key,
            'secret': secret_key,
        }

        host = 'archive.org'
        path = '/metadata/{0}'.format(self.identifier)
        http = httplib.HTTP(host)
        http.putrequest("POST", path)
        http.putheader("Host", host)
        data = urllib.urlencode(data)
        http.putheader("Content-Type", 'application/x-www-form-urlencoded')
        http.putheader("Content-Length", str(len(data)))
        http.endheaders()
        http.send(data)
        status_code, error_message, headers = http.getreply()
        resp_file = http.getfile()
        self.metadata = utils.get_item_metadata(self.identifier, self.metadata_timeout,
                                                self.secure)
        return dict(
            status_code = status_code,
            content = json.loads(resp_file.read()),
        )


    # upload_file()
    #_____________________________________________________________________________________
    def upload_file(self, local_file, remote_name=None, metadata={}, headers={},
                    queue_derive=True, ignore_bucket=False, verbose=False, debug=False):
        """Upload a single file to an item. The item will be created
        if it does not exist.

        :type local_file: str or file
        :param local_file: The filepath or file-like object to be uploaded.

        :type remote_name: str
        :param remote_name: (optional) Sets the remote filename.

        :type metadata: dict
        :param metadata: (optional) Metadata used to create a new item.

        :type headers: dict
        :param headers: (optional) Add additional IA-S3 headers to
                        request.

        :type queue_derive: bool
        :param queue_derive: (optional) Set to False to prevent an item from
                             being derived after upload.

        :type ignore_bucket: bool
        :param ignore_bucket: (optional) Set to True to ignore and
                              clobber existing files and metadata.

        :type debug: bool
        :param debug: (optional) Set to True to print headers to stdout,
                      and exit without sending the upload request.

        Usage::

            >>> import internetarchive
            >>> item = internetarchive.Item('identifier')
            >>> item.upload_file('/path/to/image.jpg',
            ...                  remote_name='photos/image1.jpg')
            True

        :rtype: bool
        :returns: True if the request was successful and file was
                  uploaded, False otherwise.

        """
        if not self.session:
            self.session = Session()

        if not hasattr(local_file, 'read'):
            local_file = open(local_file, 'rb')
        if not remote_name:
            remote_name = local_file.name.split('/')[-1]

        # Attempt to add size-hint header.    
        if not headers.get('x-archive-size-hint'):
            try:
                local_file.seek(0, os.SEEK_END)
                headers['x-archive-size-hint'] = local_file.tell()
                local_file.seek(0, os.SEEK_SET)
            except IOError:
                pass

        # Prepare Request.
        endpoint = 'http://s3.us.archive.org/{0}/{1}'.format(self.identifier, remote_name)
        headers = ias3.build_headers(metadata, headers, queue_derive=queue_derive,
                                     ignore_bucket=ignore_bucket)
        request = Request('PUT', endpoint, headers=headers)
        # TODO: Add support for multipart.
        # `contextlib.closing()` is used to make StringIO work with 
        # `with` statement.
        with closing(local_file) as data:
            request.data = data.read()
        prepped_request = request.prepare()

        if debug:
            return prepped_request 
        else:
            if verbose:
                stdout.write(' uploading file: {0}\n'.format(remote_name))
            return self.session.send(prepped_request)


    # upload()
    #_____________________________________________________________________________________
    def upload(self, files, **kwargs):
        """Upload files to an item. The item will be created if it
        does not exist.

        :type files: list
        :param files: The filepaths or file-like objects to upload.

        :type kwargs: dict
        :param kwargs: The keyword arguments from the call to
                       upload_file().

        Usage::

            >>> import internetarchive
            >>> item = internetarchive.Item('identifier')
            >>> md = dict(mediatype='image', creator='Jake Johnson')
            >>> item.upload('/path/to/image.jpg', metadata=md, queue_derive=False)
            True

        :rtype: bool
        :returns: True if the request was successful and all files were
                  uploaded, False otherwise.

        """
        def iter_directory(directory):
            for path, dir, files in os.walk(directory):
                for f in files:
                    filepath = os.path.join(path, f)
                    remote_name = os.path.relpath(filepath, directory)
                    yield (filepath, remote_name)

        if not isinstance(files, (list, tuple)):
            files = [files]

        responses = []
        for local_file in files:
            if isinstance(local_file, basestring) and os.path.isdir(local_file):
                for local_file, remote_name in iter_directory(local_file):
                    resp = self.upload_file(local_file, remote_name=remote_name, **kwargs)
                    responses.append(resp)
            else:
                resp = self.upload_file(local_file, **kwargs)
                responses.append(resp)
        return responses


# File class
#_________________________________________________________________________________________
class File(object):
    """:todo: document ``internetarchive.File`` class."""
    # init()
    #_____________________________________________________________________________________
    def __init__(self, item, file_dict):
        self.item = item
        self.external_identifier = file_dict.get('external-identifier')
        self.name = file_dict.get('name')
        self.source = file_dict.get('source')
        self.size = file_dict.get('size')
        self.size = file_dict.get('size')
        if self.size is not None:
            self.size = int(self.size)
        self.format = file_dict.get('format')
        self.mtime = file_dict.get('mtime')
        self.md5  = file_dict.get('md5')
        self.sha1 = file_dict.get('crc32')
        self.sha1 = file_dict.get('sha1')


    # __repr__()
    #_____________________________________________________________________________________
    def __repr__(self):
        return ('File(identifier={item.identifier!r}, '
                'filename={name!r}, '
                'size={size!r}, '
                'source={source!r}, '
                'format={format!r})'.format(**self.__dict__))


    # download()
    #_____________________________________________________________________________________
    def download(self, file_path=None, ignore_existing=False):
        """:todo: document ``internetarchive.File.download()`` method"""
        if file_path is None:
            file_path = self.name

        if os.path.exists(file_path) and not ignore_existing:
            raise IOError('File already exists: {0}'.format(file_path))

        parent_dir = os.path.dirname(file_path)
        if parent_dir != '' and not os.path.exists(parent_dir):
            os.makedirs(parent_dir)

        fname = self.name.encode('utf-8')
        url = '{0}/{1}'.format(self.item.download_url, urllib.quote(fname, safe=''))

        # Add cookies to request when downloading to allow privileged
        # users the ability to download access-restricted files.
        logged_in_user, logged_in_sig = config.get_cookies()
        cookies = ('logged-in-user={0}; '
                   'logged-in-sig={1}'.format(logged_in_user, logged_in_sig))

        opener = urllib2.build_opener()
        opener.addheaders.append(('Cookie', cookies))
        data = opener.open(url)
        with open(file_path, 'wb') as fp:
            fp.write(data.read())



# -*- coding: utf-8 -*-
import os, sys
inc_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, inc_path)

import internetarchive

def test_build_headers():

    metadata = {
            'collection': 'test_collection',
            'foo': u'தமிழ்',
            'subject': ['foo', 'bar', 'baz'],
            'bar': 13,
            'boo': {'test': 'dict'},
            'none': None,
            'none2': False,
            'test_foo': 'underscore',
    }
    headers = {
            'x-archive-size-hint': 19327352832,
            'x-archive-test-header': 'test value',
    }

    ias3_headers = internetarchive.ias3.build_headers(metadata, headers)

    test_output = {
            # str test.
            'x-archive-meta00-collection': 'test_collection',

            # unicode test.
            'x-archive-meta00-foo': '\xe0\xae\xa4\xe0\xae\xae\xe0\xae\xbf\xe0\xae\xb4\xe0\xaf\x8d',

            # int test
            'x-archive-meta00-bar': 13,
            
            # list test.
            'x-archive-meta00-subject':'foo',
            'x-archive-meta01-subject': 'bar',
            'x-archive-meta02-subject': 'baz',

            # convert "_" to "--" test (S3 converts "--" to "_").
            'x-archive-meta00-test--ddd': 'sdfsdf',
             
            # dict test.
            'x-archive-meta00-boo': '{"test": "dict"}',
            'x-archive-meta00-test--foo': 'underscore',

            # prepared HTTP headers test.
            'x-archive-size-hint': 19327352832,
            'x-archive-test-header': 'test value',

            # Automatically added.
            'x-archive-meta-scanner': 'Internet Archive Python library {0}'.format(internetarchive.__version__),
            'x-archive-auto-make-bucket': 1,
    }

    for key, value in ias3_headers.items():
        if key == 'Authorization':
            continue
        assert test_output[key] == value



import json

from . import __version__, config



# get_headers()
#_________________________________________________________________________________________
def build_headers(metadata={}, headers={}, auto_make_bucket=True, **kwargs):
    """Convert a dictionary of metadata into S3 compatible HTTP
    headers, and append headers to ``headers``.

    :type metadata: dict
    :param metadata: Metadata to be converted into S3 HTTP Headers
                     and appended to ``headers``.

    :type headers: dict
    :param headers: (optional) S3 compatible HTTP headers.

    """
    if not kwargs.get('access_key') or not kwargs.get('secret_key'):
        access_key, secret_key = config.get_s3_keys()
    headers['Authorization'] = 'LOW {0}:{1}'.format(access_key, secret_key)
    if auto_make_bucket:
        headers['x-archive-auto-make-bucket'] = 1
    scanner = 'Internet Archive Python library {0}'.format(__version__)
    headers['x-archive-meta-scanner'] = scanner
    if not kwargs.get('queue_derive', True):
        headers['x-archive-queue-derive'] = 0

    for meta_key, meta_value in metadata.iteritems():

        # Encode arrays into JSON strings because Archive.org does not 
        # yet support complex metadata structures in 
        # <identifier>_meta.xml.
        if isinstance(meta_value, dict):
            meta_value = json.dumps(meta_value)

        # Convert the metadata value into a list if it is not already
        # iterable.
        if not hasattr(meta_value, '__iter__'):
                meta_value = [meta_value]

        # Convert metadata items into HTTP headers and add to 
        # ``headers`` dict.
        for i, value in enumerate(meta_value):
            if not value:
                continue
            s3_header_key = 'x-archive-meta{0:02d}-{1}'.format(i, meta_key)

            # because rfc822 http headers disallow _ in names, IA-S3 will
            # translate two hyphens in a row (--) into an underscore (_).
            s3_header_key = s3_header_key.replace('_', '--')
            try:
                if not headers.get(s3_header_key):
                    headers[s3_header_key] = value.encode('utf-8')
            except AttributeError:
                if not headers.get(s3_header_key):
                    headers[s3_header_key] = value

    return dict((k,v) for k,v in headers.items())



import os, sys, shutil
inc_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, inc_path)

import internetarchive

def test_item():
    item = internetarchive.Item('stairs')
    assert item.metadata['metadata']['identifier'] == 'stairs'


def test_file():
    item = internetarchive.Item('stairs')
    filename = 'glogo.png'
    file = item.file(filename)

    assert not os.path.exists(filename)
    file.download()

    assert os.stat(filename).st_size == file.size
    os.unlink(filename)


def test_download():
    item = internetarchive.Item('stairs')
    item_dir = item.identifier
    assert not os.path.exists(item_dir)
    item.download()
    assert os.path.exists(item_dir)
    assert os.path.exists(os.path.join(item_dir, item.identifier+'_meta.xml'))
    shutil.rmtree(item_dir)



#!/usr/bin/env python
"""A command line interface for Archive.org.

usage: 
    ia [--version|--help|--debug] <command> [<args>...]

options:
    --version  
    -h, --help  
    -d, --debug  [default: True]

commands:
    help      Retrieve help for subcommands.
    configure Configure `ia`.
    metadata  Retrieve and modify metadata for items on archive.org
    upload    Upload items to archive.org
    download  Download files from archive.org
    search    Search archive.org
    mine      Download item metadata concurrently.
    catalog   Retrieve information about your catalog tasks

See 'ia help <command>' for more information on a specific command.

"""
from sys import stderr, exit
from subprocess import call

from docopt import docopt

from internetarchive import __version__



# main()
#_________________________________________________________________________________________
def main():
    """This script is the CLI driver for ia-wrapper. It dynamically
    imports and calls the subcommand specified on the command line. It 
    depends on the ``internetarchive`` and ``iacli`` packages.

    Subcommands can be arbitrarily added to the ``iacli`` package as 
    modules, and can be dynamically executed via this script, ``ia``.

    """
    args = docopt(__doc__, version=__version__, options_first=True)

    # Get subcommand.
    cmd = args['<command>']
    aliases = dict(
            md = 'metadata',
            up = 'upload',
            ca = 'catalog',
            se = 'search',
            mi = 'mine',
            do = 'download',
    )
    if cmd in aliases:
        cmd = aliases[cmd]

    argv = [cmd] + args['<args>']
    
    # Dynamically import and call subcommand module specified on the 
    # command line.
    module = 'iacli.ia_{0}'.format(cmd) 
    try:
        globals()['ia_module'] = __import__(module, fromlist=['iacli'])
    except ImportError:
        stderr.write('"{0}" is not an `ia` command!\n'.format(cmd))
        exit(1)
    if cmd == 'help':
        if not args['<args>']:
            call(['ia', '--help'])
        else:
            call(['ia', args['<args>'][0], '--help'])
    else:
        ia_module.main(argv)

if __name__ == '__main__':
    main()



try:
    from gevent import monkey, queue, spawn
    monkey.patch_all(thread=False)
except ImportError:
    raise ImportError(
    """No module named gevent

    This feature requires the gevent neworking library. gevent
    and all of it's dependencies can be installed with pip:
    \tpip install cython git+git://github.com/surfly/gevent.git@1.0rc2#egg=gevent

    """)

from internetarchive import Item



# Mine class
#_________________________________________________________________________________________
class Mine(object):
    """This class is for concurrently retrieving metadata for items on
    Archive.org.

    Usage::

        >>> import internetarchive
        >>> miner = internetarchive.Mine('itemlist.txt', workers=50)
        >>> for md in miner:
        ...     print md

    """
    # __init__()
    #_____________________________________________________________________________________
    def __init__(self, identifiers, workers=20):
        self.skips = []
        self.queue = queue
        self.workers = workers
        self.done_queueing_input = False
        self.queued_count = 0
        self.identifiers = identifiers
        self.input_queue = self.queue.JoinableQueue(1000)
        self.json_queue = self.queue.Queue(1000)


    # _metadata_getter()
    #_____________________________________________________________________________________
    def _metadata_getter(self):
        while True:
            i, identifier = self.input_queue.get()
            try:
                item = Item(identifier)
                self.json_queue.put((i, item))
            except:
                self.input_queue.put((i, identifier))
            finally:
                self.input_queue.task_done()


    # _queue_input()
    #_____________________________________________________________________________________
    def _queue_input(self):
        for i, identifier in enumerate(self.identifiers):
            self.input_queue.put((i, identifier))
            self.queued_count += 1
        self.done_queueing_input = True


    # items()
    #_____________________________________________________________________________________
    def items(self):
        spawn(self._queue_input)
        for i in range(self.workers):
            spawn(self._metadata_getter)

        def metadata_iterator_helper():
            got_count = 0
            while True:
                if self.done_queueing_input and got_count == self.queued_count:
                    break
                yield self.json_queue.get()
                got_count += 1

        return metadata_iterator_helper()



"""Retrieve and modify metadata for items on archive.org.

usage: 
    ia metadata [--help] <identifier> [--modify <key:value>... | options...]

options:
    -h, --help
    -m, --modify <key:value>   Modify the metadata of an item.
    -e, --exists               Check if an item exists. Exit status is 0 if the item
                               exists, and 1 if it does not.
    -f, --files                Return select file-level metadata.
    -F, --formats               
    -c, --files-count          Return the file-count of an item.
    -i, --item-size            Return the item-size.
    -s, --server               Return the server from which the given item is being 
                               served.
    --dir                      Return the full item-directory path.
    --d1                       Return the primary server.
    --d2                       Return the secondary server.
    -t, --target <target>      Return specified target, only.

"""
from docopt import docopt
import sys

import internetarchive
from internetarchive import get_item, modify_metadata
from iacli.argparser import get_args_dict



# main()
#_________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)
    item = internetarchive.Item(args['<identifier>'])

    # Check existence of item.
    if args['--exists']:
        if item.exists:
            sys.stdout.write('{0} exists\n'.format(item.identifier))
            sys.exit(0)
        else:
            sys.stderr.write('{0} does not exist\n'.format(item.identifier))
            sys.exit(1)

    # Modify metadata.
    elif args['--modify']:
        metadata = get_args_dict(args['--modify'])
        response = modify_metadata(args['<identifier>'], metadata)
        status_code = response['status_code']
        if not response['content']['success']:
            error_msg = response['content']['error']
            sys.stderr.write('error: {0} ({1})\n'.format(error_msg, status_code))
            sys.exit(1)
        sys.stdout.write('success: {0}\n'.format(response['content']['log']))

    # Get metadata.
    elif args['--files']:
        for f in item.files():
            files_md = [f.item.identifier, f.name, f.source, f.format, f.size, f.md5]
            sys.stdout.write('\t'.join([str(x) for x in files_md]) + '\n')
    elif args['--formats']:
        formats = set([f.format for f in item.files()])
        sys.stdout.write('\n'.join(formats))
    elif args['--files-count']:
        sys.stdout.write(str(item.metadata.get('files_count')))
    elif args['--server']:
        sys.stdout.write(str(item.metadata.get('server')))
    elif args['--dir']:
        sys.stdout.write(str(item.metadata.get('dir')))
    elif args['--d1']:
        sys.stdout.write(str(item.metadata.get('d1')))
    elif args['--d2']:
        sys.stdout.write(str(item.metadata.get('d2')))
    elif args['--item-size']:
        sys.stdout.write(str(item.metadata.get('item_size')))
    elif args['--target']:
        keys = [k.strip('/') for k in args['--target'][0].split('/') if k]
        for i, k in enumerate(keys):
            if i == 0:
                md = item.metadata.get(k)
            else:
                md = md.get(k)
        sys.stdout.write(str(md))
    else:
        sys.stdout.write(str(item.metadata))

    sys.exit(0)



# -*- coding: utf-8 -*-
from iacli.argparser import get_args_dict


def test_get_args_dict():
    test_input = [
            'collection:test_collection',
            "description: Attention: multiple colon's",
            'unicode_test:தமிழ்',
            'subject:subject1',
            'subject:subject2',
    ]
    test_output = {
            'collection': 'test_collection', 
            'description': " Attention: multiple colon's", 
            'unicode_test': '\xe0\xae\xa4\xe0\xae\xae\xe0\xae\xbf\xe0\xae\xb4\xe0\xaf\x8d', 
            'subject': ['subject1', 'subject2']
    }
    args_dict = get_args_dict(test_input)
    for key, value in args_dict.items():
        assert test_output[key] == value



from sys import stdout

from . import item, service, utils



# get_item()
#_________________________________________________________________________________________
def get_item(identifier, **kwargs): 
    return item.Item(identifier, **kwargs)


# get_metadata()
#_________________________________________________________________________________________
def get_metadata(identifier, timeout=None): 
    return utils.get_item_metadata(identifier, target='metadata', timeout=timeout)


# get_files()
#_________________________________________________________________________________________
def get_files(identifier, timeout=None): 
    return utils.get_item_metadata(identifier, target='files', timeout=timeout)


# iter_files()
#_________________________________________________________________________________________
def iter_files(identifier): 
    _item = item.Item(identifier)
    return _item.files()


# modify_metadata()
#_________________________________________________________________________________________
def modify_metadata(identifier, metadata, target='metadata'):
    _item = item.Item(identifier)
    return _item.modify_metadata(metadata, target)


# upload_file()
#_____________________________________________________________________________________
def upload_file(identifier, local_file, **kwargs):
    _item = item.Item(identifier)
    return _item.upload_file(identifier, local_file, **kwargs)
    

# upload()
#_____________________________________________________________________________________
def upload(identifier, files, **kwargs):
    """Upload files to an item. The item will be created if it
    does not exist.

    :type files: list
    :param files: The filepaths or file-like objects to upload.

    :type kwargs: dict
    :param kwargs: The keyword arguments from the call to
                   upload_file().

    Usage::

        >>> import internetarchive
        >>> item = internetarchive.Item('identifier')
        >>> md = dict(mediatype='image', creator='Jake Johnson')
        >>> item.upload('/path/to/image.jpg', metadata=md, queue_derive=False)
        True

    :rtype: bool
    :returns: True if the request was successful and all files were
              uploaded, False otherwise.

    """
    _item = item.Item(identifier)
    return _item.upload(files, **kwargs)


# download()
#_________________________________________________________________________________________
def download(identifier, **kwargs):
    """Download an item into the current working directory.

    :type concurrent: bool
    :param concurrent: Download files concurrently if ``True``.

    :type source: str
    :param source: Only download files matching given source.

    :type formats: str
    :param formats: Only download files matching the given Formats.

    :type glob_pattern: str
    :param glob_pattern: Only download files matching the given glob
                         pattern

    :type ignore_existing: bool
    :param ignore_existing: Overwrite local files if they already 
                            exist.

    :rtype: bool
    :returns: True if if files have been downloaded successfully.

    Usage::

        >>> import internetarchive
        >>> internetarchive.download('stairs', source=['metadata', 'original'])

    """
    
    _item = item.Item(identifier)
    _item.download(**kwargs)


# download_file()
#_________________________________________________________________________________________
def download_file(identifier, filename, **kwargs):
    """

    Usage::

        >>> import internetarchive
        >>> internetarchive.download_file('stairs', 'stairs.avi')

    """
    _item = item.Item(identifier)
    remote_file = _item.file(filename)
    stdout.write('downloading: {0}\n'.format(filename))
    remote_file.download(**kwargs)


# get_tasks()
#_________________________________________________________________________________________
def get_tasks(**kwargs): 
    catalog = service.Catalog(**kwargs)
    task_type = kwargs.get('task_type')
    if task_type:
        return eval('catalog.{0}_rows'.format(task_type.lower()))
    else:
        return catalog.tasks


# search()
#_________________________________________________________________________________________
def search(query, **kwargs): 
    return service.Search(query, **kwargs)


# mine()
#_________________________________________________________________________________________
def get_data_miner(identifiers, **kwargs): 
    from . import mine
    miner = mine.Mine(identifiers, **kwargs)
    return miner



from setuptools import setup
from sys import version_info, stderr, exit



extra = {}
if version_info >= (3,):
    stderr.write('Python3 is not yet supported.\n')
    exit(1)
    #extra['use_2to3'] = True
    #extra['use_2to3_fixers'] = ['your.fixers']

setup(
    name='internetarchive',
    version="0.3.9",
    author='Jacob M. Johnson',
    author_email='jake@archive.org',
    packages=['internetarchive', 'iacli'],
    entry_points = {
        'console_scripts': [
            'ia = iacli.ia:main',
        ],
    },
    url='https://github.com/jjjake/ia-wrapper',
    license='LICENSE',
    description='A python interface to archive.org.',
    long_description=open('README.rst').read(),
    install_requires=[
        'requests==2.0.0',
        'jsonpatch==1.1',
        'pytest==2.3.4',
        'docopt==0.6.1',
        'PyYAML==3.10',
    ],
    dependency_links=[
        'https://github.com/downloads/surfly/gevent/gevent-1.0rc2.tar.gz#egg=gevent-1.0.rc2',
    ],
    extras_require = {
        'speedups': [
            'ujson==1.33',
            'Cython==0.18',
            'gevent==1.0rc2',
        ],
    },
    **extra
)



"""Configure the `ia` CLI and internetarchive Python library.

usage:
    ia configure [--help] [options...]

options:
    -h, --help
    -c, --cookies  Add your IA cookies to your configuration file.
"""
from docopt import docopt
import sys
import os
import yaml



# ia_configure()
#_________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)
    sys.stdout.write(
        'Please visit https://archive.org/account/s3.php to retrieve your S3 keys\n\n')
    access_key = raw_input('Please enter your IA S3 access key: ')
    secret_key = raw_input('Please enter your IA S3 secret key: ')
    config = {
            's3': {
                'access_key': access_key,
                'secret_key': secret_key
            }
    }

    if args['--cookies']:
        config['cookies'] = {
            'logged-in-user': raw_input('Please enter your logged-in-user cookie: '),
            'logged-in-sig': raw_input('Please Enter your logged-in-sig cookie: ')
    }

    configfile = yaml.dump(config, default_flow_style=False)
    configdir = os.path.join(os.environ['HOME'], '.config')

    if not os.path.isdir(configdir) and not os.path.isfile(configdir):
        os.mkdir(configdir)
    elif os.path.isdir(configdir):
        filename = os.path.join(configdir, 'internetarchive.yml')
    else:
        filename = os.path.join(os.environ['HOME'], '.internetarchive.yml')
    if os.path.exists(filename):
        overwrite = raw_input('\nYou already have an ia config file: '
                              '{0} \n\nWould you like to overwrite it?'
                              '[y/n] '.format(filename).lower())
        if overwrite not in ['y', 'yes']:
            sys.stdout.write('\nExiting without overwriting config file!\n')
            sys.exit(1)
    with open(filename, 'wb') as fp:
        os.chmod(filename, 0o700)
        fp.write(configfile)

    sys.stdout.write('\nSuccessfully saved your new config to: {0}\n'.format(filename))
    sys.exit(0)



#!/usr/bin/env python

"""Test uploading through the internetarchive python package.

This test script creates a new archive.org item, and therefore is named in
such a way that py.test does not automatically run it.
"""

import internetarchive as ia
import datetime
import hashlib
import os
import StringIO
import tempfile
import time


# get_new_item()
#_________________________________________________________________________________________
def get_new_item():
    """return an ia item object for an item that does not yet exist"""
    now = datetime.datetime.utcnow()
    item_name = 'test_upload_iawrapper_' + now.strftime('%Y_%m_%d_%H%M%S')
    item = ia.Item(item_name)
    if item.exists is False:
        return item

    raise KeyError, 'Could not find a unique item name after 5 tries'


# get_file()
#_________________________________________________________________________________________
def get_file(item_name, file_name):
    """get a file from a newly-created item. Wait for file to land in item, retry if needed"""

    for i in range(5):
        print '  waiting 30 seconds for upload of', file_name
        time.sleep(30)

        item = ia.Item(item_name)
        f = item.file(file_name)
        if f is not None:
            return f

    raise KeyError, 'Could not retrieve file after 5 tries'


# upload_stringIO()
#_________________________________________________________________________________________
def upload_stringIO(item):
    contents = 'hello world'
    name = 'hello_world.txt'

    fh = StringIO.StringIO(contents)
    fh.name = name

    item.upload(fh, metadata=dict(collection='test_collection'))

    f = get_file(item.identifier, name)
    assert f.sha1 == hashlib.sha1(contents).hexdigest()


# upload_tempfile()
#_________________________________________________________________________________________
def upload_tempfile(item):
    contents = 'temporary file contents'
    temp_file = tempfile.NamedTemporaryFile(suffix='.txt')

    temp_file.write(contents)
    temp_file.seek(0, os.SEEK_SET)

    item.upload(temp_file.name, metadata=dict(collection='test_collection'))
    temp_file.close()

    f = get_file(item.identifier, os.path.basename(temp_file.name))
    assert f.sha1 == hashlib.sha1(contents).hexdigest()


# test_upload()
#_________________________________________________________________________________________
def test_upload():
    print 'Finding new item name'
    item = get_new_item()

    print 'Uploading new item named', item.identifier

    print 'Testing upload using StringIO'
    upload_stringIO(item)

    print 'Testing upload using tempfile'
    upload_tempfile(item)

    print 'Finished upload test'


# main()
#_________________________________________________________________________________________
if __name__ == '__main__':
    if os.environ.get('AWS_ACCESS_KEY_ID') is None:
        raise LookupError, 'You must set AWS_ACCESS_KEY_ID environment variable!'

    if os.environ.get('AWS_SECRET_ACCESS_KEY') is None:
        raise LookupError, 'You must set AWS_SECRET_ACCESS_KEY environment variable!'

    test_upload()
